# coding: utf-8
# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------
from azure.appconfiguration.aio import AzureAppConfigurationClient
from azure.core.async_paging import AsyncItemPaged
from azure.core.exceptions import ResourceExistsError
from testcase import AppConfigTestCase
from typing import List


class AsyncAppConfigTestCase(AppConfigTestCase):
    def create_aad_client(self, appconfiguration_endpoint_string):
        cred = self.get_credential(AzureAppConfigurationClient, is_async=True)
        return AzureAppConfigurationClient(appconfiguration_endpoint_string, cred)

    def create_client(self, appconfiguration_connection_string):
        return AzureAppConfigurationClient.from_connection_string(appconfiguration_connection_string)

    async def add_for_test(self, client, config_setting):
        try:
            await client.add_configuration_setting(config_setting)
        except ResourceExistsError:
            pass

    async def convert_to_list(self, items: AsyncItemPaged) -> List:
        list = []
        async for item in items:
            list.append(item)
        return list

    async def set_up(self, appconfiguration_string, is_aad=False):
        if is_aad:
            self.client = self.create_aad_client(appconfiguration_string)
        else:
            self.client = self.create_client(appconfiguration_string)
        await self.add_for_test(self.client, self.create_config_setting())
        await self.add_for_test(self.client, self.create_config_setting_no_label())

    async def tear_down(self):
        if self.client is not None:
            config_settings = self.client.list_configuration_settings()
            async for config_setting in config_settings:
                await self.client.delete_configuration_setting(key=config_setting.key, label=config_setting.label)
            await self.client.close()
        else:
            raise ValueError("Client is None!")



# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------

from azure.core.pipeline.transport import HttpRequest, HttpResponse
from azure.core.pipeline import PipelineRequest, PipelineResponse
from azure.appconfiguration._sync_token import SyncToken, SyncTokenPolicy


def test_parse_sync_token():
    test_sync_token = "jtqGc1I4=MDoyOA==;sn=28"
    sync_token = SyncToken.from_sync_token_string(test_sync_token)
    assert sync_token.token_id == "jtqGc1I4"
    assert sync_token.value == "MDoyOA=="
    assert sync_token.sequence_number == 28


def test_save_sync_token():
    test_sync_token = "jtqGc1I4=MDoyOA==;sn=28"
    header = {"Sync-Token": test_sync_token}
    request = HttpRequest("GET", "https://bing.com/")
    response = HttpResponse(request, None)
    response.headers = header
    pipeline_response = PipelineResponse(request, response, None)
    sync_token_policy = SyncTokenPolicy()
    sync_token_policy.on_response(None, pipeline_response)
    sync_token = sync_token_policy._sync_tokens["jtqGc1I4"]
    assert sync_token.token_id == "jtqGc1I4"
    assert sync_token.value == "MDoyOA=="
    assert sync_token.sequence_number == 28


def test_set_sync_token():
    test_sync_token = "jtqGc1I4=MDoyOA==;sn=28"
    header = {"Sync-Token": test_sync_token}
    request = HttpRequest("GET", "https://bing.com/")
    response = HttpResponse(request, None)
    response.headers = header
    pipeline_response = PipelineResponse(request, response, None)
    pipeline_request = PipelineRequest(request, None)
    sync_token_policy = SyncTokenPolicy()
    sync_token_policy.on_response(None, pipeline_response)
    sync_token_policy.on_request(pipeline_request)
    sync_token_header = pipeline_request.http_request.headers.get("Sync-Token")
    assert sync_token_header == "jtqGc1I4=MDoyOA=="


def test_save_multi_sync_token():
    test_sync_token = "syncToken1=val1;sn=6,syncToken2=val2;sn=10"
    header = {"Sync-Token": test_sync_token}
    request = HttpRequest("GET", "https://bing.com/")
    response = HttpResponse(request, None)
    response.headers = header
    pipeline_response = PipelineResponse(request, response, None)
    sync_token_policy = SyncTokenPolicy()
    sync_token_policy.on_response(None, pipeline_response)
    sync_token = sync_token_policy._sync_tokens["syncToken1"]
    assert sync_token.token_id == "syncToken1"
    assert sync_token.value == "val1"
    assert sync_token.sequence_number == 6
    sync_token = sync_token_policy._sync_tokens["syncToken2"]
    assert sync_token.token_id == "syncToken2"
    assert sync_token.value == "val2"
    assert sync_token.sequence_number == 10


def test_set_multi_sync_token():
    test_sync_token = "syncToken1=val1;sn=6,syncToken2=val2;sn=10"
    header = {"Sync-Token": test_sync_token}
    request = HttpRequest("GET", "https://bing.com/")
    response = HttpResponse(request, None)
    response.headers = header
    pipeline_response = PipelineResponse(request, response, None)
    pipeline_request = PipelineRequest(request, None)
    sync_token_policy = SyncTokenPolicy()
    sync_token_policy.on_response(None, pipeline_response)
    sync_token_policy.on_request(pipeline_request)
    sync_token_header = pipeline_request.http_request.headers.get("Sync-Token")
    assert "syncToken1=val1" in sync_token_header
    assert "syncToken2=val2" in sync_token_header


def test_update_cached_sync_token():
    test_sync_token = "syncToken1=val1;sn=6"
    header = {"Sync-Token": test_sync_token}
    request = HttpRequest("GET", "https://bing.com/")
    response = HttpResponse(request, None)
    response.headers = header
    pipeline_response = PipelineResponse(request, response, None)
    sync_token_policy = SyncTokenPolicy()
    sync_token_policy.on_response(None, pipeline_response)
    sync_token = sync_token_policy._sync_tokens["syncToken1"]
    assert sync_token.token_id == "syncToken1"
    assert sync_token.value == "val1"
    assert sync_token.sequence_number == 6
    test_new_sync_token = "syncToken1=val2;sn=10"
    header["Sync-Token"] = test_new_sync_token
    response.headers = header
    pipeline_response = PipelineResponse(request, response, None)
    sync_token_policy.on_response(None, pipeline_response)
    sync_token = sync_token_policy._sync_tokens["syncToken1"]
    assert sync_token.token_id == "syncToken1"
    assert sync_token.value == "val2"
    assert sync_token.sequence_number == 10



# coding: utf-8

# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------

"""
FILE: list_revision_sample.py

DESCRIPTION:
    This sample demos how to get configuration setting revision history synchronously.

USAGE: python list_revision_sample.py

    Set the environment variables with your own values before running the sample:
    1) APPCONFIGURATION_CONNECTION_STRING: Connection String used to access the Azure App Configuration.
"""
import os
from azure.appconfiguration import AzureAppConfigurationClient, ConfigurationSetting


def main():
    CONNECTION_STRING = os.environ["APPCONFIGURATION_CONNECTION_STRING"]

    # Create an app config client
    client = AzureAppConfigurationClient.from_connection_string(CONNECTION_STRING)

    config_setting = ConfigurationSetting(
        key="MyKey", value="my value", content_type="my content type", tags={"my tag": "my tag value"}
    )
    returned_config_setting = client.set_configuration_setting(config_setting)

    returned_config_setting.value = "new value"
    returned_config_setting.content_type = "new content type"
    client.set_configuration_setting(config_setting)

    # [START list_revisions]
    items = client.list_revisions(key_filter="MyKey", tags_filter=["my tag=my tag value"])
    for item in items:
        print(item)
    # [END list_revisions]

    client.delete_configuration_setting(key="MyKey")


if __name__ == "__main__":
    main()



# ------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# -------------------------------------------------------------------------

from datetime import datetime
from typing import Optional, Tuple, Dict, Any

# Connection string component prefixes
_ENDPOINT_PREFIX = "Endpoint="
_ID_PREFIX = "Id="
_SECRET_PREFIX = "Secret="


def parse_connection_string(connection_string: str) -> Tuple[str, str, str]:
    # connection_string looks like Endpoint=https://xxxxx;Id=xxxxx;Secret=xxxx
    segments = connection_string.split(";")
    if len(segments) != 3:
        raise ValueError("Invalid connection string.")

    endpoint = ""
    id_ = ""
    secret = ""
    for segment in segments:
        segment = segment.strip()
        if segment.startswith(_ENDPOINT_PREFIX):
            endpoint = str(segment[len(_ENDPOINT_PREFIX) :])
        elif segment.startswith(_ID_PREFIX):
            id_ = str(segment[len(_ID_PREFIX) :])
        elif segment.startswith(_SECRET_PREFIX):
            secret = str(segment[len(_SECRET_PREFIX) :])
        else:
            raise ValueError("Invalid connection string.")

    if not endpoint or not id_ or not secret:
        raise ValueError("Invalid connection string.")

    return endpoint, id_, secret


def get_current_utc_time() -> str:
    return str(datetime.utcnow().strftime("%b, %d %Y %H:%M:%S.%f ")) + "GMT"


def get_key_filter(*args: Optional[str], **kwargs: Any) -> Tuple[Optional[str], Dict[str, Any]]:
    key_filter = None
    if len(args) > 0:
        key_filter = args[0]
        if "key_filter" in kwargs:
            raise TypeError(
                "AzureAppConfigurationClient.list_configuration_settings() got multiple values for argument "
                "'key_filter'"
            )
    return key_filter or kwargs.pop("key_filter", None), kwargs


def get_label_filter(*args: Optional[str], **kwargs: Any) -> Tuple[Optional[str], Dict[str, Any]]:
    label_filter = None
    if len(args) > 1:
        label_filter = args[1]
        if "label_filter" in kwargs:
            raise TypeError(
                "AzureAppConfigurationClient.list_configuration_settings() got multiple values for argument "
                "'label_filter'"
            )
    return label_filter or kwargs.pop("label_filter", None), kwargs



# coding: utf-8

# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------

"""
FILE: snapshot_sample_async.py

DESCRIPTION:
    This sample demos how to create/retrieve/archive/recover/list configuration settings snapshot and list configuration settings of a snapshot asynchronously.

USAGE: python snapshot_sample_async.py

    Set the environment variables with your own values before running the sample:
    1) APPCONFIGURATION_CONNECTION_STRING: Connection String used to access the Azure App Configuration.
"""
import asyncio
import os
from azure.appconfiguration import ConfigurationSetting
from azure.appconfiguration.aio import AzureAppConfigurationClient
from uuid import uuid4


async def main():
    CONNECTION_STRING = os.environ["APPCONFIGURATION_CONNECTION_STRING"]
    config_setting1 = ConfigurationSetting(key="my_key1", label="my_label1")
    config_setting2 = ConfigurationSetting(key="my_key1", label="my_label2")
    snapshot_name = str(uuid4())
    async with AzureAppConfigurationClient.from_connection_string(CONNECTION_STRING) as client:
        await client.add_configuration_setting(config_setting1)
        await client.add_configuration_setting(config_setting2)

        from azure.appconfiguration import ConfigurationSettingsFilter

        filters = [ConfigurationSettingsFilter(key="my_key1", label="my_label1")]
        response = await client.begin_create_snapshot(name=snapshot_name, filters=filters)
        created_snapshot = await response.result()
        print(created_snapshot)
        print("")

        received_snapshot = await client.get_snapshot(name=snapshot_name)
        print(received_snapshot)
        print("")

        archived_snapshot = await client.archive_snapshot(name=snapshot_name)
        print(archived_snapshot)
        print("")

        recovered_snapshot = await client.recover_snapshot(name=snapshot_name)
        print(recovered_snapshot)
        print("")

        snapshots = client.list_snapshots()
        async for snapshot in snapshots:
            print(snapshot)
        print("")

        config_settings = client.list_configuration_settings(snapshot_name=snapshot_name)
        async for config_setting in config_settings:
            print(config_setting)

        await client.delete_configuration_setting(key=config_setting1.key, label=config_setting1.label)
        await client.delete_configuration_setting(key=config_setting2.key, label=config_setting2.label)


if __name__ == "__main__":
    asyncio.run(main())



# ------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# -------------------------------------------------------------------------

import hashlib
import base64
import hmac
from urllib.parse import urlparse
from azure.core.credentials import AzureKeyCredential
from azure.core.pipeline.policies import HTTPPolicy
from ._utils import get_current_utc_time


class AppConfigRequestsCredentialsPolicy(HTTPPolicy):
    """Implementation of request-oauthlib except and retry logic."""

    def __init__(self, credential: AzureKeyCredential, endpoint: str, id_credential: str):
        super(AppConfigRequestsCredentialsPolicy, self).__init__()
        self._credential = credential
        self._endpoint = endpoint
        self._id_credential = id_credential

    def _signed_request(self, request):
        verb = request.http_request.method.upper()

        parsed_url = urlparse(self._endpoint)
        # Get the path, query and hash fragment from url, which looks like https://host/path?query#fragment
        query_url = str(request.http_request.url[len(parsed_url.geturl()) :])
        # Need URL() to get a correct encoded key value, from "%2A" to "*", when transport is in type AioHttpTransport.
        # There's a similar scenario in azure-storage-blob, the check logic is from there.
        try:
            from yarl import URL
            from azure.core.pipeline.transport import (  # pylint:disable=non-abstract-transport-import,no-name-in-module
                AioHttpTransport,
            )

            if (
                isinstance(request.context.transport, AioHttpTransport)
                or isinstance(getattr(request.context.transport, "_transport", None), AioHttpTransport)
                or isinstance(
                    getattr(getattr(request.context.transport, "_transport", None), "_transport", None),
                    AioHttpTransport,
                )
            ):
                query_url = str(URL(query_url))
        except (ImportError, TypeError):
            pass
        signed_headers = "x-ms-date;host;x-ms-content-sha256"

        utc_now = get_current_utc_time()
        if request.http_request.body is None:
            request.http_request.body = ""
        content_digest = hashlib.sha256((request.http_request.body.encode("utf-8"))).digest()
        content_hash = base64.b64encode(content_digest).decode("utf-8")

        string_to_sign = verb + "\n" + query_url + "\n" + utc_now + ";" + parsed_url.netloc + ";" + content_hash

        decoded_secret = base64.b64decode(self._credential.key)
        digest = hmac.new(decoded_secret, string_to_sign.encode("utf-8"), hashlib.sha256).digest()
        signature = base64.b64encode(digest).decode("utf-8")

        signature_header = {
            "x-ms-date": utc_now,
            "x-ms-content-sha256": content_hash,
            "Authorization": "HMAC-SHA256 Credential="
            + self._id_credential
            + "&SignedHeaders="
            + signed_headers
            + "&Signature="
            + signature,
        }

        request.http_request.headers.update(signature_header)

        return request

    def send(self, request):
        self._signed_request(request)
        return self.next.send(request)



# coding: utf-8

# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------

"""
FILE: conditional_operation_sample.py

DESCRIPTION:
    This sample demos how to conditional set/get/delete configuration settings synchronously.

USAGE: python conditional_operation_sample.py

    Set the environment variables with your own values before running the sample:
    1) APPCONFIGURATION_CONNECTION_STRING: Connection String used to access the Azure App Configuration.
"""
import os
from azure.core import MatchConditions
from azure.core.exceptions import ResourceModifiedError
from azure.appconfiguration import AzureAppConfigurationClient, ConfigurationSetting


def main():
    CONNECTION_STRING = os.environ["APPCONFIGURATION_CONNECTION_STRING"]

    # Create an app config client
    client = AzureAppConfigurationClient.from_connection_string(CONNECTION_STRING)

    # Unconditional set
    config_setting = ConfigurationSetting(
        key="MyKey", value="my value", content_type="my content type", tags={"my tag": "my tag value"}
    )
    client.set_configuration_setting(config_setting)

    # Unconditional get
    first_get = client.get_configuration_setting(key="MyKey")
    if first_get is None:
        return print("Error, unconditional set failed.")
    print(first_get)

    # Conditional get, expect to return None because it is not modified
    second_get = client.get_configuration_setting(
        key="MyKey", etag=first_get.etag, match_condition=MatchConditions.IfModified
    )
    print(second_get)

    # Conditional set
    first_get.value = "new value"
    client.set_configuration_setting(configuration_setting=first_get, match_condition=MatchConditions.IfNotModified)

    # Conditional set, expect to see error because it is modified
    try:
        client.set_configuration_setting(configuration_setting=first_get, match_condition=MatchConditions.IfNotModified)
    except ResourceModifiedError:
        pass

    client.delete_configuration_setting(key="MyKey")


if __name__ == "__main__":
    main()



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
# pylint: disable=wrong-import-position

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ._patch import *  # pylint: disable=unused-wildcard-import


from ._models import (  # type: ignore
    Error,
    Key,
    KeyValue,
    KeyValueFilter,
    Label,
    OperationDetails,
    Snapshot,
    SnapshotUpdateParameters,
)

from ._enums import (  # type: ignore
    ConfigurationSettingFields,
    LabelFields,
    OperationState,
    SnapshotComposition,
    SnapshotFields,
    SnapshotStatus,
)
from ._patch import __all__ as _patch_all
from ._patch import *
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "Error",
    "Key",
    "KeyValue",
    "KeyValueFilter",
    "Label",
    "OperationDetails",
    "Snapshot",
    "SnapshotUpdateParameters",
    "ConfigurationSettingFields",
    "LabelFields",
    "OperationState",
    "SnapshotComposition",
    "SnapshotFields",
    "SnapshotStatus",
]
__all__.extend([p for p in _patch_all if p not in __all__])  # pyright: ignore
_patch_sdk()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import os

from devtools_testutils.perfstress_tests import PerfStressTest

from azure.appconfiguration import ConfigurationSetting, AzureAppConfigurationClient as SyncAppConfigClient
from azure.appconfiguration.aio import AzureAppConfigurationClient as AsyncAppConfigClient


class GetTest(PerfStressTest):
    def __init__(self, arguments):
        super().__init__(arguments)
        connection_string = self.get_from_env("AZURE_APP_CONFIG_CONNECTION_STRING")
        self.key = "KEY"
        self.service_client = SyncAppConfigClient.from_connection_string(connection_string=connection_string)
        self.async_service_client = AsyncAppConfigClient.from_connection_string(connection_string=connection_string)

    async def global_setup(self):
        await super().global_setup()
        kv = ConfigurationSetting(
            key=self.key,
            value="VALUE",
        )
        await self.async_service_client.set_configuration_setting(kv)

    async def close(self):
        # await self.async_service_client.close()
        await super().close()

    def run_sync(self):
        self.service_client.get_configuration_setting(key=self.key)

    async def run_async(self):
        await self.async_service_client.get_configuration_setting(key=self.key)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import os
import uuid

from devtools_testutils.perfstress_tests import PerfStressTest

from azure.appconfiguration import ConfigurationSetting, AzureAppConfigurationClient as SyncAppConfigClient
from azure.appconfiguration.aio import AzureAppConfigurationClient as AsyncAppConfigClient


class SetTest(PerfStressTest):
    service_client = None
    async_service_client = None

    def __init__(self, arguments):
        super().__init__(arguments)
        connection_string = self.get_from_env("AZURE_APP_CONFIG_CONNECTION_STRING")
        self.key = "KEY"
        self.value = str(uuid.uuid4())
        self.service_client = SyncAppConfigClient.from_connection_string(connection_string=connection_string)
        self.async_service_client = AsyncAppConfigClient.from_connection_string(connection_string=connection_string)

    async def close(self):
        # await self.async_service_client.close()
        await super().close()

    def run_sync(self):
        kv = ConfigurationSetting(
            key=self.key,
            value="VALUE" + self.value,
        )
        self.service_client.set_configuration_setting(kv)

    async def run_async(self):
        kv = ConfigurationSetting(
            key=self.key,
            value="VALUE" + self.value,
        )
        await self.async_service_client.set_configuration_setting(kv)
